//https://www.lawebdelprogramador.com/foros/Java/1672399-Ayuda-con-Java.html

package entregas;

import java.io.Serializable;

import javax.swing.JOptionPane;


/**
 * Una <b>Entrega</b> alberga todos los datos referentes a un envío de paquetería:<br>
 * informacion del consignatario, informadion del destino, tipo servicio, fechas...<br>
 * Todos los datos son opcionales a la hora de crear una <b>Entrega</b>. Las únicas restricciones son:<br>
 * <ul><li><i>Numero de acuse</i> ha de comenzar por una letra, seguida de una cantidad indeterminada de números.</li>
 * <li>Las <i>fechas</i> han de usar el formato --> <i>dd/mm/aaaa</i></li></ul><br>
 * Se puede crear una <b>Entrega</b> sin <i>número de acuse</i>, pero luego no será posible agregarla a una <b>Coleccion de
 * Entregas</b>, ya que este campo es utilizado como clave identificativa.<br>
 * Esta clase implementa la interfaz <b><i>Serializable</i></b> para que las <b>Colecciones de Entregas</b> puedan 
 * ser guardadas y leidas en un archivo de bytes. 
 * @author Kabuto
 *
 */
@SuppressWarnings("serial")
public final class Entrega implements Serializable{

	private String numAcuse;//formato: A1234567
	private String nombreConsig;
	private String direccion;
	private String estado;
	private String tipoServicio;
	private String cantiPaquetes;
	private String nomDestinatario;
	private String direcDestinatario;
	private String municipioDestinatario;
	//private String firmaRecibi;
	private String fecha;
	private String observaciones;
	private String bajaSistema;
	private String fechaEnvioDestino;
	private String folioEnvioDestino;
	
	public Entrega(String nAcuse, String nConsig, String dire, String estd, String tServicio, String cntPaq,
			String nDesti, String direcDesti, String muniDesti, String fech, String obs, String bjSis, String fEnvDest,
			String foliEnvDest) {
		numAcuse = nAcuse.toUpperCase();
		nombreConsig = nConsig;
		direccion = dire;
		estado = estd;
		tipoServicio = tServicio;
		cantiPaquetes = cntPaq;
		nomDestinatario = nDesti;
		direcDestinatario = direcDesti;
		municipioDestinatario = muniDesti;
		fecha = fech;
		observaciones = obs;
		bajaSistema = bjSis;
		fechaEnvioDestino = fEnvDest;
		folioEnvioDestino = foliEnvDest;
	}
	
	public String getNumAcuse() {
		return numAcuse;
	}
	
	/**
	 * Establece el número de acuse, pero solo si respeta el formato exigido.<br>
	 * Ha de comenzar por una letra y a continuacion una cantidad indeterminada de 
	 * numeros.<br>Si no se respeta el formato, avisara por pantalla de que el dato 
	 * ha sido rechazado.
	 * @param numAcuse <b>String</b> con el numero de acuse a establecer
	 * @return <b>true</b> si el dato ha sido aceptado, <b>false</b> si fue rechazado.
	 */
	public boolean setNumAcuse(String numAcuse) {
		if (numAcuse.toUpperCase().matches("[A-Z]{1}\\d+"))
		{
			this.numAcuse = numAcuse.toUpperCase();
			return true;
		}
		else
		{
			JOptionPane.showMessageDialog(null, "Numero acuse invalido.\n"
					+ "Ha de comenzar por una letra seguida de una serie de numeros", "Error validando  acuse",
					JOptionPane.WARNING_MESSAGE);
			return false;
		}
	}
	public String getNombreConsig() {
		return nombreConsig;
	}
	public void setNombreConsig(String nombreConsig) {
		this.nombreConsig = nombreConsig;
	}
	public String getDireccion() {
		return direccion;
	}
	public void setDireccion(String direccion) {
		this.direccion = direccion;
	}
	public String getEstado() {
		return estado;
	}
	public void setEstado(String estado) {
		this.estado = estado;
	}
	public String getTipoServicio() {
		return tipoServicio;
	}
	public void setTipoServicio(String tipoServicio) {
		this.tipoServicio = tipoServicio;
	}
	public String getCantiPaquetes() {
		return cantiPaquetes;
	}
	public void setCantiPaquetes(String cantiPaquetes) {
		this.cantiPaquetes = cantiPaquetes;
	}
	public String getNomDestinatario() {
		return nomDestinatario;
	}
	public void setNomDestinatario(String nomDestinatario) {
		this.nomDestinatario = nomDestinatario;
	}
	public String getDirecDestinatario() {
		return direcDestinatario;
	}
	public void setDirecDestinatario(String direcDestinatario) {
		this.direcDestinatario = direcDestinatario;
	}
	public String getMunicipioDestinatario() {
		return municipioDestinatario;
	}
	public void setMunicipioDestinatario(String municipioDestinatario) {
		this.municipioDestinatario = municipioDestinatario;
	}
	/*
	public String getFirmaRecibi() {
		return firmaRecibi;
	}
	public void setFirmaRecibi(String firmaRecibi) {
		this.firmaRecibi = firmaRecibi;
	}*/
	public String getFecha() {
		return fecha;
	}
	
	/**
	 * Establece una fecha valida. Esta puede estar vacia, pero si contiene algo
	 * ha de cumplir el formato dd/mm/aaaa.
	 * @param fecha <b>String</b> con fecha a validar
	 * @return <b>true</b> si la fecha se acepto, <b>false</b> de lo contrario.
	 */
	public boolean setFecha(String fecha) {
		if (fecha.matches("\\d{1,2}/\\d{1,2}/\\d{4}") || fecha.isEmpty())
		{
			this.fecha = fecha;
			return true;
		}
		else
		{
			JOptionPane.showMessageDialog(null, "Formato fecha invalido.\n"
					+ "USe formato dd/mm/aaaa", "Error validando fecha",
					JOptionPane.WARNING_MESSAGE);
			return false;
		}
	}
	public String getObservaciones() {
		return observaciones;
	}
	public void setObservaciones(String observaciones) {
		this.observaciones = observaciones;
	}
	public String getBajaSistema() {
		return bajaSistema;
	}
	public void setBajaSistema(String bajaSistema) {
		this.bajaSistema = bajaSistema;
	}
	public String getFechaEnvioDestino() {
		return fechaEnvioDestino;
	}
	
	/**
	 * Establece una fecha valida. Esta puede estar vacia, pero si contiene algo
	 * ha de cumplir el formato dd/mm/aaaa.
	 * @param fechaEnvioDestino <b>String</b> con fecha a validar
	 * @return <b>true</b> si la fecha se acepto, <b>false</b> de lo contrario.
	 */
	public boolean setFechaEnvioDestino(String fechaEnvioDestino) {
		if (fecha.matches("\\d{1,2}/\\d{1,2}/\\d{4}"))
		{
			this.fechaEnvioDestino = fechaEnvioDestino;
			return true;
		}
		else
		{
			JOptionPane.showMessageDialog(null, "Formato fecha invalido.\n"
					+ "USe formato dd/mm/aaaa", "Error validando fecha",
					JOptionPane.WARNING_MESSAGE);
			return false;
		}
	}
	public String getFolioEnvioDestino() {
		return folioEnvioDestino;
	}
	public void setFolioEnvioDestino(String folioEnvioDestino) {
		this.folioEnvioDestino = folioEnvioDestino;
	}
	
	/**
	 * Dos Entregas se consideran iguales si coinciden sus numeros de acuse.
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Entrega)
			return numAcuse.equals(((Entrega)(obj)).getNumAcuse());
		else
			return false;
	}
	
	/**
	 * El código <i>hash</i> se genera a partir del <i>número de acuse</i> ya 
	 * que es el atributo clave para distinguir una <b>Entrega</b> de otras.
	 */
	@Override
	public int hashCode() {
		return numAcuse.hashCode();
	}
	
}
