package paneles;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;
import javax.swing.border.EmptyBorder;

@SuppressWarnings("serial")
/**
 * Modela el panel superior de la interfaz donde se muestra información sobre la <b>Colección</b> actual
 * y se ofrecen botones para recorrer las <b>Entregas</b> registradas.
 * @author Kabuto
 */
public final class Norte extends JPanel{
	
	private MiLabel et_avisoCambios = new MiLabel("<html><p align=\"center\">No hay cambios<br>que guardar</p></html>");
	private MiLabel et_nombreColecActual = new MiLabel("No hay coleccion");
	private MiLabel et_marcador = new MiLabel(null);
	public MiBoton bt_anterior = new MiBoton("Anterior");
	public MiBoton bt_siguiente  = new MiBoton("Siguiente");
	private int totalRegistros = 0;
	private int registroActual = 0;
	
	public Norte() {
		setBorder(new EmptyBorder(5, 5, 5, 5));
		actualizaAvisoCambios(true);
		actualizaEtMarcador();
		bt_anterior.setEnabled(false);
		bt_siguiente.setEnabled(false);
		et_nombreColecActual.setForeground(Color.RED);
		//setLayout(new FlowLayout(FlowLayout.CENTER, 20, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		add(panelCambios());
		add(panelNombre());
		add(panelMarcador());
		add(Box.createRigidArea(new Dimension(10, 0)));
		add(panelBotones());
	}
	
	/**
	 * Recibe los datos de una nueva <b>Coleccion</b> para actualizarse.<br>
	 * El origen de la <b>Coleccion</b> puede ser totalmente nueva o de una
	 * almacenada en disco.
	 * @param datos Array de tipo <b>Object</b> con:<br><ul>
	 * <li><b>String</b> con nombre de la <b>Colección</b></li>
	 * <li><b>Integer</b> con cantidad de registros</li></ul>
	 */
	public void nuevaColeccion(Object[] datos) {
		et_nombreColecActual.setText((String)datos[0]);
		et_nombreColecActual.setForeground(new Color(34, 153, 84));
		Integer registros = (Integer)datos[1];
		if (registros == 0) {
			totalRegistros = 0;
			registroActual = 0;
			activarBotones(false, false);
		}
		else {
			totalRegistros = registros;
			registroActual = 1;
			if (totalRegistros == 1)
				activarBotones(false, false);
			else
				activarBotones(false, true);
		}
		actualizaEtMarcador();
	}
	
	/**
	 * Cambia el texto que informa de si hay cambios pendientes de guardar.
	 * @param cambiosGuardados <b>boolean</b> para saber si los cambios están
	 * guardados o no.<br>Este dato lo proporciona el <b>Modelo</b>
	 */
	public void actualizaAvisoCambios(boolean cambiosGuardados) {
		if (cambiosGuardados) {
			//et_avisoCambios.setText("<html><p align=\"center\">No hay cambios<br>que guardar</p></html>");
			et_avisoCambios.setText("Cambios guardados: SI");
			et_avisoCambios.setForeground(new Color(13, 75, 202));
		}
		else {
			//et_avisoCambios.setText("<html><p align=\"center\">Si hay cambios<br>para guardar</p></html>");
			et_avisoCambios.setText("Cambios guardados: NO");
			et_avisoCambios.setForeground(Color.RED);
		}
	}
	
	/**
	 * Actualiza el marcador que informa de la posicion del registro actual y de la cantidad
	 * de registros totales.<br>Además decide que botones estarán activos.
	 * @param datosMarcador Array con dos enteros: posicion actual y registros totales
	 */
	public void actualizaMarcador(int[] datosMarcador) {
		totalRegistros = datosMarcador[0];
		registroActual = datosMarcador[1];
		if (totalRegistros > 1)
		{
			if (registroActual == totalRegistros)
				activarBotones(true, false);
			else if (registroActual == 1)
				activarBotones(false, true);
			else
				activarBotones(true, true);
		}
		actualizaEtMarcador();
	}
	
	
	//Metodos privados
	/**
	 * Activa o desactiva los dos botones del panel.
	 * @param anterior <b>boolean</b> para activar/desactivar el boton "Anterior" 
	 * @param siguiente <b>boolean</b> para activar/desactivar el boton "Siguiente"
	 */
	private void activarBotones(boolean anterior, boolean siguiente) {
		bt_anterior.setEnabled(anterior);
		bt_siguiente.setEnabled(siguiente);
	}
	
	/**
	 * Actualiza las etiquetas del marcador con los valores actuales.<br>
	 * Los dos numeros enteros están formateados para ocupar 4 cifras cada uno.
	 */
	private void actualizaEtMarcador() {
		et_marcador.setText(String.format("Entrega: %4d/%4d", registroActual, totalRegistros));
	}
	
	/**
	 * Construye el subpanel que informa de si los cambios están guardados.
	 * @return <b>JPanel</b>
	 */
	private JPanel panelCambios() {
		et_avisoCambios.setPreferredSize(new Dimension(190, 17));
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
		panel.add(et_avisoCambios);
		return panel;
	}
	
	/**
	 * Construye el subpanel que informa del nombre de la <b>Colección</b> actual
	 * @return <b>JPanel</b>
	 */
	private JPanel panelNombre() {
		JPanel panel = new JPanel();
		//panel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
		MiLabel txt = new MiLabel("Coleccion de Entregas actual: ");
		SpringLayout layout = new SpringLayout();
		panel.setLayout(layout);
		panel.add(txt);
		//panel.add(new MiLabel("Coleccion de Entregas actual: "));
		panel.add(et_nombreColecActual);
		layout.putConstraint(SpringLayout.WEST, txt,
                30,
                SpringLayout.WEST, panel);
		layout.putConstraint(SpringLayout.WEST, et_nombreColecActual,
                270,
                SpringLayout.WEST, panel);
		layout.putConstraint(SpringLayout.NORTH, txt,
                5,
                SpringLayout.NORTH, panel);
		layout.putConstraint(SpringLayout.NORTH, et_nombreColecActual,
                5,
                SpringLayout.NORTH, panel);
		return panel;
	}
	
	/**
	 * Construye el subpanel que informa del registro actual y del total 
	 * de registros existentes.
	 * @return <b>JPanel</b>
	 */
	private JPanel panelMarcador() {
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout());
		panel.add(et_marcador);
		return panel;
	}
	
	/**
	 * Construye el subpanel con los botones "Anterior" y "Siguiente".
	 * @return <b>JPanel</b>
	 */
	private JPanel panelBotones() {
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));
		panel.add(bt_anterior);
		panel.add(bt_siguiente);
		return panel;
	}
	
	//Clases internas
	/**
	 * Clase que hereda de <b>JLabel</b> y que usa una fuente de texto
	 * predeterminada.
	 * @author Kabuto
	 */
	private class MiLabel extends JLabel {
		public MiLabel(String txt) {
			super(txt);
			setFont(new Font("Verdana", Font.BOLD, 14));
		}
	}
	
	/**
	 * Clase que hereda de <b>JButton</b> y que usa una fuente de texto
	 * predeterminada.
	 * @author Kabuto
	 */
	public class MiBoton extends JButton {
		public MiBoton(String texto) {
			super(texto);
			setFont(new Font("Verdana", Font.BOLD, 14));
			this.setFocusPainted(false);
		}
	}

}
