package paneles;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;

@SuppressWarnings("serial")
/**
 * Modela el panel inferior de la interfaz, donde se ofrecen botones para gestionar las <b>Colecciones</b>
 * y <b>Entregas</b>
 * @author Kabuto
 */
public final class Sur extends JPanel{
	
	public MiBoton bt_nuevaColeccion = new MiBoton("<html><p align=\"center\">Nueva<br>Coleccion</p></html>");
	public MiBoton bt_abrirColeccion = new MiBoton("<html><p align=\"center\">Abrir<br>Coleccion</p></html>");
	public MiBoton bt_guardarColeccion = new MiBoton("<html><p align=\"center\">Guardar<br>Coleccion</p></html>");
	public MiBoton bt_nuevaEntrega = new MiBoton("<html><p align=\"center\">Nueva<br>Entrega</p></html>");
	public MiBoton bt_modiEntrega = new MiBoton("<html><p align=\"center\">Modificar<br>Entrega</p></html>");
	public MiBoton bt_guardarEntrega = new MiBoton("<html><p align=\"center\">Guardar<br>Entrega</p></html>");
	public MiBoton bt_eliminarEntrega = new MiBoton("<html><p align=\"center\">Eliminar<br>Entrega</p></html>");
	
	public Sur() {
		activarBotones(false, false, false, false, false);
		setLayout(new FlowLayout(FlowLayout.CENTER, 20, 10));
		add(bt_nuevaColeccion);
		add(bt_abrirColeccion);
		add(bt_guardarColeccion);
		JSeparator sep = new JSeparator(SwingConstants.VERTICAL);
		sep.setPreferredSize(new Dimension(4, 60));
		add(sep);
		add(bt_nuevaEntrega);
		add(bt_modiEntrega);
		add(bt_guardarEntrega);
		add(bt_eliminarEntrega);
	}
	
	/**
	 * Establece que botones estarán activos y cuales no.<br>Estas activaciones depende de la
	 * situación actual. Por ejemplo, el botón "Eliminar" solo estará activo si hay una <b>Entrega</b>
	 * en pantalla, pero no cuando no haya <b>Colección</b> o esta esté vacía.<br>
	 * Los botones "Nueva Coleccion" y "Abrir Coleccion" siempre estarán activos.
	 * @param guardaColec <b>boolean</b> para activar/desactivar el botón "Guardar Coleccion"
	 * @param NuevaEntre <b>boolean</b> para activar/desactivar el botón "Nueva Entrega"
	 * @param modificar <b>boolean</b> para activar/desactivar el botón "Modificar Entrega"
	 * @param guardar <b>boolean</b> para activar/desactivar el botón "Guardar Entrega"
	 * @param eliminar <b>boolean</b> para activar/desactivar el botón "Eliminar Entrega"
	 */
	public void activarBotones(boolean guardaColec, boolean NuevaEntre, boolean modificar, boolean guardar, boolean eliminar) {
		bt_guardarColeccion.setEnabled(guardaColec);
		bt_nuevaEntrega.setEnabled(NuevaEntre);
		bt_modiEntrega.setEnabled(modificar);
		bt_guardarEntrega.setEnabled(guardar);
		bt_eliminarEntrega.setEnabled(eliminar);
	}
	
	/**
	 * Clase que hereda de <b>JButton</b> y que usa una fuente de texto
	 * predeterminada.<br>
	 * Además agrega un boolean para que algunos botones puedan tener dos modos de comportamiento distintos.<br>
	 * Por ejemplo, el botón "Modificar Entrega", cuando <i>modo1 = true</i> activa el proceso de modificar una <b>Entrega</b> y entonces
	 * pasa a <i>modo1 = false.</i>. En este modo, si se le pulsa lo que hace es cancelar el proceso de modificación y retomar el modo
	 * de funcionamiento primario.
	 * @author Kabuto
	 */
	public class MiBoton extends JButton {
		public boolean modo1 = true;//Este atributo lo usan los botones modificar y nueva entrega, porque tienen 2 comportamientos distintos
		public MiBoton(String texto) {
			super(texto);
			setFont(new Font("Verdana", Font.BOLD, 18));
			this.setFocusPainted(false);
		}
	}
	
}
